local AddName, AddonTable = ...
local L = AddonTable.L

------------------------------
--		Options Equates		--
------------------------------
local Both, Enhanced, Base, MainHand, OffHand = 3, 1, 2, 1, 2
local SameLevel, BossLevel, Auto = 1, 2, 3

----------------------------------
--		Text Return Formats		--
----------------------------------
local Miss_Chance_Format = { "%.2f%%", "%.2f%%", "%.2f/%.2f%%", }

------------------------------
--		Global Updater		--
------------------------------

AddonTable.speed, AddonTable.lowDmg, AddonTable.hiDmg, AddonTable.critRanged, AddonTable.gunEquipped, AddonTable.hitRanged, AddonTable.posBuff, AddonTable.negBuff = 0, 0, 0, 0, 0, 0, 0, 0

function AddonTable.RangedUpdate()

 	AddonTable.targetLevel = AddonTable.playerLevel + 3
	if UnitCanAttack("player", "target") then
		AddonTable.targetLevel = UnitLevel("target")
		if AddonTable.targetLevel <= 0 then
			AddonTable.targetLevel = AddonTable.playerLevel + 3
		end
	end

	-- Attack Speed
	AddonTable.speed, AddonTable.lowDmg, AddonTable.hiDmg, AddonTable.posBuff, AddonTable.negBuff = UnitRangedDamage("player")
	
	-- Critical
	AddonTable.critRanged = GetRangedCritChance("player")
	AddonTable.gunEquipped = IsEquippedItemType("Guns")
	AddonTable.critRanged = AddonTable.critRanged + AddonTable.totemWrath
	
	-- Dwarf racial [Crit]
	if AddonTable.gunEquipped and AddonTable.dwarfRacial then
		AddonTable.critRanged = AddonTable.critRanged + 1
	end	
	
	-- Hit
	local hitModifier = GetHitModifier()
	AddonTable.hitRanged = GetCombatRatingBonus(CR_HIT_RANGED)
	
	if AddonTable.hitRanged == nil then
		AddonTable.hitRanged = 0
	end
	if hitModifier == nil then
		hitModifier = 0
	end
	
	AddonTable.hitRanged = AddonTable.hitRanged + hitModifier + AddonTable.heroicPresence	

end

-- Attack Power
function AddonTable.FunctionList.AP(HUD, data, options, ...)

	local base, posBuff, negBuff = UnitRangedAttackPower("player")
	local effectiveAP = base + posBuff + negBuff
	local debuffColor = ""
	
	if negBuff > 0 then
		debuffColor = "|cffC41E3A"
	end
	
	HUD:UpdateText(data, debuffColor .. effectiveAP)
end	
------------------------------------------------

-- Critical
function AddonTable.FunctionList.RangedSpeed(HUD, data, options, ...)

	local critRangedChance = GetRangedCritChance("player")
	
	HUD:UpdateText(data, ("%.2f%%"):format(critRangedChance))
end
------------------------------------------------

-- Damage
function AddonTable.FunctionList.RangedDMG(HUD, data, options, ...)

	local EB, enhancedStat, baseStat = options.Max_Average_Damage
	local speed, lowDmg, hiDmg, posBuff, negBuff, percent = UnitRangedDamage("player")
	local avgDmg = (hiDmg + lowDmg) / 2

	local debuffColor = ""
	local returnText
	
	 if negBuff < 0 then		
		debuffColor = "|cffC41E3A"
	end

	if AddonTable.Band(EB, Max) then
		returnText = debuffColor .. ("%.0f"):format(hiDmg)
	end
	if  AddonTable.Band(EB, Average) then
		returnText = ("%.0f"):format(avgDmg)
	end
	if AddonTable.Band(EB, Both) then
		returnText = debuffColor .. ("%.0f"):format(hiDmg) .. "/" .. ("%.0f"):format(avgDmg)
	end
	
	if returnText == nil then
		returnText = ""
	end	

	HUD:UpdateText(data, returnText)
end
------------------------------------------------

-- Attack Speed
function AddonTable.FunctionList.rDPS(HUD, data, options, ...)

	local speed = UnitRangedDamage("player");
	
	HUD:UpdateText(data, ("%.1f"):format(speed))
end
------------------------------------------------

-- Critcal
function AddonTable.FunctionList.RangedCrit(HUD, data, options, ...)

	local EB, enhancedStat, baseStat = options.Percent_Rating
	local statFormat = Percent_Rating_Format[EB]
	local critRating = GetCombatRating(CR_CRIT_RANGED)
	
	if AddonTable.Band(EB, Percentage) then
		enhancedStat = AddonTable.critRanged
	end
	if  AddonTable.Band(EB, Rating) then
		baseStat = critRating
	end
	
	HUD:UpdateText(data, format(statFormat, enhancedStat and enhancedStat or baseStat, baseStat))
end
------------------------------------------------

-- Hit
function AddonTable.FunctionList.RangedHit(HUD, data, options, ...)
	
	local label, hitRangedChance, percentAdded, hitRangedmod = "rHit: ", 0, "", 0
	hitRangedChance = GetHitModifier("player")
	
	if (AddonTable.classFilename == "HUNTER") then
		local slotId = GetInventorySlotInfo("RangedSlot")
		local link = GetInventoryItemLink("player", slotId)
		
		if link then
			hitRangedChance = GetHitModifier("player");
				if hitRangedChance == nil then
					hitRangedChance = 0
				end		
			local itemId, enchantId = link:match("item:(%d+):(%d+)")
				if enchantId then
					enchantId = tonumber(enchantId)
						if enchantId == 2523 then
							hitRangedmod = 3
							hitRangedChance = hitRangedChance + 3
						end
				end
		end	
	HUD:UpdateText(data, (hitRangedChance and hitRangedChance) .. "%")
	
	else
	
	if hitRangedChance == nil then
		hitRangedChance = 0
	end
	
	HUD:UpdateText(data, (hitRangedChance .. "%"))
end
------------------------------------------------

-- Miss
function AddonTable.FunctionList.RangedMiss(HUD, data, options, ...)

	local EB, enhancedStat, baseStat = options.Level_Same_Boss
	local statFormat = Double_Percent_Format[EB]
	
	local rangedMissLevel = 0
	local rangedMissBoss = 0
	local rangedAttackBase, rangedAttackMod = UnitRangedAttack("player")
	local levelDiffDef = AddonTable.targetLevel - AddonTable.playerLevel
	local levelDefense = AddonTable.playerLevel * 5
	local bossDefense = (AddonTable.playerLevel + levelDiffDef) * 5
	local rWeaponSkill = (rangedAttackBase + rangedAttackMod)
	local rangedSkillDiff = levelDefense - rWeaponSkill
	local rbossSkillDiff = bossDefense - rWeaponSkill

	if rangedSkillDiff > 10 then	
		rangedMissLevel = (7 + ((rangedSkillDiff - 10) * 0.4))
		rangedMissLevel = rangedMissLevel - AddonTable.hitRanged	
	elseif rangedSkillDiff <= 10 then	
		rangedMissLevel = (5 + ((rangedSkillDiff) * 0.1))
		rangedMissLevel = rangedMissLevel - AddonTable.hitRanged
	end

	if rbossSkillDiff > 10 then
		rangedMissBoss = (7 + ((rbossSkillDiff - 10) * 0.4))
		rangedMissBoss = rangedMissBoss - AddonTable.hitRanged	
	elseif rbossSkillDiff <= 10 then
		rangedMissBoss = (5 + ((rbossSkillDiff) * 0.1))
		rangedMissBoss = rangedMissBoss - AddonTable.hitRanged	
	end 

	if rangedMissLevel < 0 then
		rangedMissLevel = 0
	elseif rangedMissLevel > 100 then
		rangedMissLevel = 100
	end

	if rangedMissBoss < 0 then
		rangedMissBoss = 0
	elseif rangedMissBoss > 100 then
		rangedMissBoss = 100
	end

	if AddonTable.Band(EB, SameLevel) then
		enhancedStat = rangedMissLevel
	end
	if  AddonTable.Band(EB, BossLevel) then
		baseStat = rangedMissBoss
	end
	
	HUD:UpdateText(data, format(statFormat, enhancedStat and enhancedStat or baseStat, baseStat))
end
------------------------------------------------

